@@ -183,6 +183,18 @@ setMethod("genericExpFunConvert", signature("MxExpectationRAM"),
 				omxQuotes(modelname), "does not contain colnames")
 			stop(msg, call. = FALSE)
 		}
+		sMatrix <- flatModel[[sMatrix]]
+		if (!is.null(sMatrix)) {
+			if (!is(sMatrix, "MxAlgebra") &&
+			    all(diag(sMatrix$values) == 0) && all(diag(sMatrix$free) == FALSE) &&
+			    all(is.na(diag(sMatrix$labels)))) {
+				msg <- paste("The S matrix associated",
+				"with the RAM expectation function in model", 
+				omxQuotes(modelname), "is fixed to zero on the",
+				"diagonal. Your model must allow some variance.")
+				stop(msg, call.=FALSE)
+			}
+		}
 		mMatrix <- flatModel[[mMatrix]]		
 		if (hasMeanModel && !is.null(mMatrix)) {
 			means <- dimnames(mMatrix)
