@@ -100,7 +100,6 @@ nameGDOptimizerConstraintOutput <- function(paramNames, constraintNames, GDstep,
 		#Initialize variables:
 		cfvNames <- NULL
 		lmNames <- NULL
-		cjr <- 1
 		constraintRows <- NULL
 		constraintCols <- NULL
 		
@@ -119,7 +118,6 @@ nameGDOptimizerConstraintOutput <- function(paramNames, constraintNames, GDstep,
 				for(co in 1:constraintCols[i]){
 					for(ro in 1:constraintRows[i]){
 						cfvNames <- c(cfvNames, paste(constraintNames[i],"[",ro,",",co,"]",sep=""))
-						cjr <- cjr+1
 					}
 				}
 			}
@@ -155,6 +153,62 @@ nameGDOptimizerConstraintOutput <- function(paramNames, constraintNames, GDstep,
 			output$istate <- GDstep@output$istate
 		}
 	}
+	else if(GDstep@engine=="SLSQP"){
+		
+		#Initialize variables:
+		cfvNames <- NULL
+		constraintRows <- NULL
+		constraintCols <- NULL
+		
+		#Filter extraneous elements and generate vectors of names:
+		paramNames <- paramNames[!(paramNames %in% GDstep@.excludeVars)] #<--Probably not necessary...
+		if(length(constraintNames) && length(GDstep@output$constraintRows && length(GDstep@output$constraintCols))){
+			emptyConstraints <- (GDstep@output$constraintRows==0 | GDstep@output$constraintCols==0)
+			#Assuming that "empty" constraints have no function values...
+			constraintNames <- constraintNames[!emptyConstraints]
+			constraintRows <- GDstep@output$constraintRows[!emptyConstraints]
+			constraintCols <- GDstep@output$constraintCols[!emptyConstraints]
+		}
+		if(length(constraintNames) && length(constraintRows) && length(constraintCols)){
+			for(i in 1:length(constraintNames)){
+				for(co in 1:constraintCols[i]){
+					for(ro in 1:constraintRows[i]){
+						cfvNames <- c(cfvNames, paste(constraintNames[i],"[",ro,",",co,"]",sep=""))
+					}
+				}
+			}
+		}
+		
+		#Assign names to components, and components to 'output' list:
+		if(length(GDstep@output$constraintFunctionValues)){
+			if(length(GDstep@output$constraintFunctionValues)==length(cfvNames)){
+				names(GDstep@output$constraintFunctionValues) <- cfvNames
+			}
+			output$constraintFunctionValues <- GDstep@output$constraintFunctionValues
+		}
+		if(length(GDstep@output$constraintJacobian)){
+			if(ncol(GDstep@output$constraintJacobian) && ncol(GDstep@output$constraintJacobian)==length(paramNames)){
+				colnames(GDstep@output$constraintJacobian) <- paramNames
+			}
+			if(nrow(GDstep@output$constraintJacobian) && nrow(GDstep@output$constraintJacobian)==length(cfvNames)){
+				rownames(GDstep@output$constraintJacobian) <- cfvNames
+			}
+			output$constraintJacobian <- GDstep@output$constraintJacobian
+		}
+		if(length(GDstep@output$LagrangeMultipliers)){
+			if(length(GDstep@output$LagrangeMultipliers)==length(cfvNames)){
+				names(GDstep@output$LagrangeMultipliers) <- cfvNames
+			}
+			output$LagrangeMultipliers <- GDstep@output$LagrangeMultipliers
+		}
+		if(length(GDstep@output$LagrHessian)){
+			if(ncol(GDstep@output$LagrHessian) && ncol(GDstep@output$LagrHessian)==length(paramNames) && 
+				 nrow(GDstep@output$LagrHessian)==length(paramNames)){
+				dimnames(GDstep@output$LagrHessian) <- list(paramNames,paramNames)
+			}
+			output$LagrHessian <- GDstep@output$LagrHessian
+		}
+	}
 	return(output)
 }
 
