@@ -9,6 +9,8 @@
 #include "finiteDifferences.h"
 
 #include "nlopt.h"
+#include "slsqp.h"
+#include "nlopt-internal.h"
 
 #ifdef SHADOW_DIAG
 #pragma GCC diagnostic warning "-Wshadow"
@@ -16,21 +18,21 @@
 
 namespace SLSQP {
 
-	struct context {
-		GradientOptimizerContext &goc;
-		int origeq;
-		int eqredundent;
-		std::vector<bool> eqmask;
-		context(GradientOptimizerContext &_goc) : goc(_goc) {
-			eqredundent = 0;
-		};
+struct context {
+	GradientOptimizerContext &goc;
+	int origeq;
+	int eqredundent;
+	std::vector<bool> eqmask;
+	context(GradientOptimizerContext &_goc) : goc(_goc) {
+		eqredundent = 0;
 	};
+};
 
 struct fit_functional {
 	GradientOptimizerContext &goc;
-
+	
 	fit_functional(GradientOptimizerContext &_goc) : goc(_goc) {};
-
+	
 	double operator()(double *x, int thrId) const {
 		int mode = 0;
 		return goc.evalFit(x, thrId, &mode);
@@ -58,7 +60,7 @@ static double nloptObjectiveFunction(unsigned n, const double *x, double *grad,
 		return nan("infeasible");
 	}
 	if (!grad) return fit;
-
+	
 	Eigen::Map< Eigen::VectorXd > Epoint((double*) x, n);
 	Eigen::Map< Eigen::VectorXd > Egrad(grad, n);
 	if (goc->getWanted() & FF_COMPUTE_GRADIENT) {
@@ -70,8 +72,8 @@ static double nloptObjectiveFunction(unsigned n, const double *x, double *grad,
 		if (goc->verbose >= 3) mxLog("fd_gradient start");
 		fit_functional ff(*goc);
 		gradient_with_ref(goc->gradientAlgo, goc->numOptimizerThreads,
-				  goc->gradientIterations, goc->gradientStepSize,
-				  ff, fit, Epoint, Egrad);
+                    goc->gradientIterations, goc->gradientStepSize,
+                    ff, fit, Epoint, Egrad);
 		goc->grad = Egrad;
 	}
 	if (goc->verbose >= 3) {
@@ -82,9 +84,9 @@ static double nloptObjectiveFunction(unsigned n, const double *x, double *grad,
 
 struct equality_functional {
 	GradientOptimizerContext &goc;
-
+	
 	equality_functional(GradientOptimizerContext &_goc) : goc(_goc) {};
-
+	
 	template <typename T1, typename T2>
 	void operator()(Eigen::MatrixBase<T1> &x, Eigen::MatrixBase<T2> &result) const {
 		goc.copyFromOptimizer(x.derived().data());
@@ -105,19 +107,19 @@ static void nloptEqualityFunction(unsigned m, double* result, unsigned n, const
 	if (grad) {
 		goc.eqNorm = Eresult.array().abs().sum();
 		fd_jacobian(goc.gradientAlgo, goc.gradientIterations, goc.gradientStepSize,
-			    ff, Eresult, Epoint, jacobian);
+              ff, Eresult, Epoint, jacobian);
 		if (ctx.eqmask.size() == 0) {
 			ctx.eqmask.assign(m, false);
 			for (int c1=0; c1 < int(m-1); ++c1) {
 				for (int c2=c1+1; c2 < int(m); ++c2) {
 					bool match = (Eresult[c1] == Eresult[c2] &&
-						      (jacobian.col(c1) == jacobian.col(c2)));
+                   (jacobian.col(c1) == jacobian.col(c2)));
 					if (match && !ctx.eqmask[c2]) {
 						ctx.eqmask[c2] = true;
 						++ctx.eqredundent;
 						if (goc.verbose >= 2) {
 							mxLog("nlopt: eq constraint %d is redundent with %d",
-							      c1, c2);
+             c1, c2);
 						}
 					}
 				}
@@ -135,7 +137,7 @@ static void nloptEqualityFunction(unsigned m, double* result, unsigned n, const
 			if (ctx.eqredundent) {
 				if (goc.verbose >= 1) {
 					mxLog("nlopt: detected %d redundent equality constraints; retrying",
-					      ctx.eqredundent);
+           ctx.eqredundent);
 				}
 				nlopt_opt opt = (nlopt_opt) goc.extraData;
 				nlopt_force_stop(opt);
@@ -160,9 +162,9 @@ static void nloptEqualityFunction(unsigned m, double* result, unsigned n, const
 
 struct inequality_functional {
 	GradientOptimizerContext &goc;
-
+	
 	inequality_functional(GradientOptimizerContext &_goc) : goc(_goc) {};
-
+	
 	template <typename T1, typename T2>
 	void operator()(Eigen::MatrixBase<T1> &x, Eigen::MatrixBase<T2> &result) const {
 		goc.copyFromOptimizer(x.derived().data());
@@ -185,7 +187,7 @@ static void nloptInequalityFunction(unsigned m, double *result, unsigned n, cons
 	if (grad) {
 		goc->ineqNorm = Eresult.array().abs().sum();
 		fd_jacobian(goc->gradientAlgo, goc->gradientIterations, goc->gradientStepSize,
-			    ff, Eresult, Epoint, jacobian);
+              ff, Eresult, Epoint, jacobian);
 		if (!std::isfinite(Eresult.sum())) {
 			// infeasible at start of major iteration
 			nlopt_opt opt = (nlopt_opt) goc->extraData;
@@ -197,6 +199,60 @@ static void nloptInequalityFunction(unsigned m, double *result, unsigned n, cons
 	}
 }
 
+static void omxExtractSLSQPConstraintInfo(nlopt_slsqp_wdump wkspc, nlopt_opt opt, GradientOptimizerContext &goc){
+	int n = opt->n;
+	int  n1 = n+1;
+	int M = wkspc.M; //<--Total number of constraints (i.e. constraint function elements, not MxConstraint objects)
+	int* lengths = wkspc.lengths;
+	double* realwkspc = wkspc.realwkspc;
+	int i=0, ro=0, co=0;
+	
+	Eigen::MatrixXd Lmat;
+	Eigen::MatrixXd Dmat;
+	Lmat.setZero(n,n);
+	Dmat.setZero(n,n);
+	
+	/*SLSQP Jacobian, starting at beginning of realwkspc, is in column-major order; uninitialized "padding" comes after valid elements.
+	Jacobian starting at realwkspc[sum(lengths[0:5])] is in row-major order and has no padding(?)*/
+	goc.constraintJacobianOut.resize(M,n);
+	for(i=0; i<M*n; i++){
+		goc.constraintJacobianOut(ro,co) = realwkspc[i];
+		ro++;
+		if(ro==M){
+			ro = 0;
+			co++;
+		}
+	}
+	ro=0;
+	co=0;
+	
+	goc.constraintFunValsOut.resize(lengths[1]);
+	for(i=0; i<lengths[1]; i++){
+		goc.constraintFunValsOut[i] = realwkspc[lengths[0]+i];
+	}
+	
+	int Wst = lengths[0]+lengths[1]+lengths[2]+lengths[3]+lengths[4]+lengths[5]+lengths[6];
+	
+	goc.LagrMultipliersOut.resize(M);
+	for(i=0; i<M; i++){
+		goc.LagrMultipliersOut[i] = realwkspc[Wst+i];
+	}
+	
+	for(i=M; i<M+(n*n1/2); i++){
+		if(ro==co){
+			Dmat(ro,co) = realwkspc[Wst+i];
+			Lmat(ro,co) = 1.0;
+		}
+		else{Lmat(ro,co) = realwkspc[Wst+i];}
+		ro++;
+		if(ro==n){
+			co++;
+			ro = co;
+		}
+	}
+	goc.LagrHessianOut = Lmat * Dmat * Lmat.transpose();
+}
+
 };
 
 void omxInvokeNLOPT(GradientOptimizerContext &goc)
@@ -204,49 +260,55 @@ void omxInvokeNLOPT(GradientOptimizerContext &goc)
 	double *est = goc.est.data();
 	goc.optName = "SLSQP";
 	goc.setupSimpleBounds();
-
+	
 	int oldWanted = goc.getWanted();
 	goc.setWanted(0);
 	omxState *globalState = goc.getState();
-    
-        nlopt_opt opt = nlopt_create(NLOPT_LD_SLSQP, goc.numFree);
+	
+	nlopt_opt opt = nlopt_create(NLOPT_LD_SLSQP, goc.numFree);
 	goc.extraData = opt;
-        //local_opt = nlopt_create(NLOPT_LD_SLSQP, n); // Subsidiary algorithm
-        
-        //nlopt_set_local_optimizer(opt, local_opt);
-        nlopt_set_lower_bounds(opt, goc.solLB.data());
-        nlopt_set_upper_bounds(opt, goc.solUB.data());
-
+	//local_opt = nlopt_create(NLOPT_LD_SLSQP, n); // Subsidiary algorithm
+	
+	//nlopt_set_local_optimizer(opt, local_opt);
+	nlopt_set_lower_bounds(opt, goc.solLB.data());
+	nlopt_set_upper_bounds(opt, goc.solUB.data());
+	
 	int eq, ieq;
 	globalState->countNonlinearConstraints(eq, ieq, false);
-
+	
 	// The *2 is there to roughly equate accuracy with NPSOL.
 	nlopt_set_ftol_rel(opt, goc.ControlTolerance * 2);
 	nlopt_set_ftol_abs(opt, std::numeric_limits<double>::epsilon());
-        
+	
 	nlopt_set_min_objective(opt, SLSQP::nloptObjectiveFunction, &goc);
-
+	
 	double feasibilityTolerance = Global->feasibilityTolerance;
 	SLSQP::context ctx(goc);
-        if (eq + ieq) {
+	if (eq + ieq) {
 		ctx.origeq = eq;
-                if (ieq > 0){
+		if (ieq > 0){
 			if (goc.verbose >= 2) mxLog("%d inequality constraints", ieq);
 			goc.inequality.resize(ieq);
 			std::vector<double> tol(ieq, feasibilityTolerance);
 			nlopt_add_inequality_mconstraint(opt, ieq, SLSQP::nloptInequalityFunction, &goc, tol.data());
-                }
-                
-                if (eq > 0){
+		}
+		
+		if (eq > 0){
 			if (goc.verbose >= 2) mxLog("%d equality constraints", eq);
 			goc.equality.resize(eq);
 			std::vector<double> tol(eq, feasibilityTolerance);
 			nlopt_add_equality_mconstraint(opt, eq, SLSQP::nloptEqualityFunction, &ctx, tol.data());
-                }
+		}
 	}
-        
+	
+	//The following four lines are only sensible if using SLSQP (noted in case we ever use a different optimizer from NLOPT):
+	struct nlopt_slsqp_wdump wkspc;
+	//wkspc.lengths = (int*)calloc(8, sizeof(int));
+	wkspc.realwkspc = (double*)calloc(1, sizeof(double)); //<--Just to initialize it; it'll be resized later.
+	opt->work = (nlopt_slsqp_wdump*)&wkspc;
+	
 	int priorIterations = goc.getIteration();
-
+	
 	double fit = 0;
 	int code = nlopt_optimize(opt, est, &fit);
 	if (ctx.eqredundent) {
@@ -254,16 +316,18 @@ void omxInvokeNLOPT(GradientOptimizerContext &goc)
 		eq -= ctx.eqredundent;
 		std::vector<double> tol(eq, feasibilityTolerance);
 		nlopt_add_equality_mconstraint(opt, eq, SLSQP::nloptEqualityFunction, &ctx, tol.data());
-
+		
 		code = nlopt_optimize(opt, est, &fit);
 	}
-
+	
 	if (goc.verbose >= 2) mxLog("nlopt_optimize returned %d", code);
-
-        nlopt_destroy(opt);
-
+	SLSQP::omxExtractSLSQPConstraintInfo(wkspc, opt, goc);
+	opt->work = NULL;
+	
+	nlopt_destroy(opt);
+	
 	goc.setWanted(oldWanted);
-
+	
 	if (code == NLOPT_INVALID_ARGS) {
 		Rf_error("NLOPT invoked with invalid arguments");
 	} else if (code == NLOPT_OUT_OF_MEMORY) {
@@ -279,9 +343,9 @@ void omxInvokeNLOPT(GradientOptimizerContext &goc)
 			goc.informOut = INFORM_NONLINEAR_CONSTRAINTS_INFEASIBLE;
 		} else if (goc.getIteration() - priorIterations <= 2) {
 			Rf_error("%s: Failed due to singular matrix E or C in LSQ subproblem or "
-				 "rank-deficient equality constraint subproblem or "
-				 "positive directional derivative in line search "
-				 "(eq %.4g ineq %.4g)", goc.optName, goc.eqNorm, goc.ineqNorm);
+              "rank-deficient equality constraint subproblem or "
+              "positive directional derivative in line search "
+              "(eq %.4g ineq %.4g)", goc.optName, goc.eqNorm, goc.ineqNorm);
 		} else {
 			goc.informOut = INFORM_NOT_AT_OPTIMUM;  // is this correct? TODO
 		}
@@ -293,3 +357,4 @@ void omxInvokeNLOPT(GradientOptimizerContext &goc)
 		goc.informOut = INFORM_CONVERGED_OPTIMUM;
 	}
 }
+
