@@ -4,14 +4,20 @@
 #include <stdlib.h>
 #include <time.h>
 #include <stdbool.h>
+#include "omxDefines.h"
+#include <Eigen/Core>
+#include <Eigen/Dense>
 #include "matrix.h"
 #include "omxCsolnp.h"
-#include <Eigen/Dense>
 //#include <iostream>
 //#include <iomanip>
 //using std::cout;
 //using std::endl;
 
+#ifdef SHADOW_DIAG
+#pragma GCC diagnostic warning "-Wshadow"
+#endif
+
 struct CSOLNP {
 
     int flag, flag_NormgZ, minr_rec;
@@ -107,7 +113,7 @@ void CSOLNP::solnp(double *solPars, int verbose)
     
     fit.myineqFun();
     Eigen::RowVectorXd ineqv_e(fit.inequality.size());
-    ineqv_e= fit.inequality;
+    ineqv_e= -fit.inequality;
 
     Eigen::MatrixXd hessv_e;
     
@@ -162,9 +168,9 @@ void CSOLNP::solnp(double *solPars, int verbose)
     }
     
     double rho   = fit.ControlRho;
-    int maxit = fit.ControlMajorLimit;
+    int maxit = fit.maxMajorIterations;
     int minit = fit.ControlMinorLimit;
-    double delta = fit.ControlFuncPrecision;
+    double delta = fit.gradientStepSize;
     double tol   = fit.ControlTolerance;
     
     int tc = nineq + neq;
@@ -180,7 +186,7 @@ void CSOLNP::solnp(double *solPars, int verbose)
     
     fit.myineqFun();
     Eigen::RowVectorXd ineqv_e(nineq);
-    ineqv_e= fit.inequality;
+    ineqv_e= -fit.inequality;
     
     Eigen::MatrixXd lambda_e;
     
@@ -253,7 +259,7 @@ void CSOLNP::solnp(double *solPars, int verbose)
     eqv_e = fit.equality;
         
     fit.myineqFun();
-    ineqv_e = fit.inequality;
+    ineqv_e = -fit.inequality;
     
     obj_constr_eval(funvMatrix_e, eqv_e, ineqv_e, ob_e, verbose);
 
@@ -309,7 +315,7 @@ void CSOLNP::solnp(double *solPars, int verbose)
             fit.solEqBFun();
             eqv_e = fit.equality;
             fit.myineqFun();
-            ineqv_e = fit.inequality;
+            ineqv_e = -fit.inequality;
             obj_constr_eval(funvMatrix_e, eqv_e, ineqv_e, ob_e, verbose);
 
             if ( ind[indHasEq] > 0){
@@ -357,7 +363,7 @@ void CSOLNP::solnp(double *solPars, int verbose)
         eqv_e = fit.equality;
         funvMatrix_e[0] = funv;
         fit.myineqFun();
-        ineqv_e = fit.inequality;
+        ineqv_e = -fit.inequality;
 
         obj_constr_eval(funvMatrix_e, eqv_e, ineqv_e, ob_e, verbose);
         
@@ -689,7 +695,7 @@ void CSOLNP::subnp(Eigen::MatrixBase<T2>& pars, Eigen::MatrixBase<T1>& yy_e, Eig
             
             Eigen::RowVectorXd funv_e(1); funv_e[0] = funv;
             Eigen::RowVectorXd eqv_e(neq); eqv_e = fit.equality;
-            Eigen::RowVectorXd ineqv_e(nineq); ineqv_e= fit.inequality;
+            Eigen::RowVectorXd ineqv_e(nineq); ineqv_e= -fit.inequality;
             
             obj_constr_eval(funv_e, eqv_e, ineqv_e, firstPart_e, verbose);
             
@@ -860,7 +866,7 @@ void CSOLNP::subnp(Eigen::MatrixBase<T2>& pars, Eigen::MatrixBase<T1>& yy_e, Eig
         Eigen::MatrixXd firstPart_e(1, 1 + neq + nineq);
         Eigen::RowVectorXd funv_e(1); funv_e[0] = funv;
         Eigen::RowVectorXd eqv_e(neq); eqv_e = fit.equality;
-        Eigen::RowVectorXd ineqv_e(nineq); ineqv_e= fit.inequality;
+        Eigen::RowVectorXd ineqv_e(nineq); ineqv_e= -fit.inequality;
         
         obj_constr_eval(funv_e, eqv_e, ineqv_e, firstPart_e, verbose);
         
@@ -925,7 +931,7 @@ void CSOLNP::subnp(Eigen::MatrixBase<T2>& pars, Eigen::MatrixBase<T1>& yy_e, Eig
                 Eigen::MatrixXd firstPart_e(1, 1 + neq + nineq);
                 Eigen::RowVectorXd funv_e(1); funv_e[0] = funv;
                 Eigen::RowVectorXd eqv_e(neq); eqv_e = fit.equality;
-                Eigen::RowVectorXd ineqv_e(nineq); ineqv_e= fit.inequality;
+                Eigen::RowVectorXd ineqv_e(nineq); ineqv_e= -fit.inequality;
                 
                 obj_constr_eval(funv_e, eqv_e, ineqv_e, firstPart_e, verbose);
                 
@@ -1144,7 +1150,7 @@ void CSOLNP::subnp(Eigen::MatrixBase<T2>& pars, Eigen::MatrixBase<T1>& yy_e, Eig
         Eigen::MatrixXd firstPart_e(1, 1 + neq + nineq);
         Eigen::RowVectorXd funv_e(1); funv_e[0] = funv;
         Eigen::RowVectorXd eqv_e(neq); eqv_e = fit.equality;
-        Eigen::RowVectorXd ineqv_e(nineq); ineqv_e= fit.inequality;
+        Eigen::RowVectorXd ineqv_e(nineq); ineqv_e= -fit.inequality;
         
         obj_constr_eval(funv_e, eqv_e, ineqv_e, firstPart_e, verbose);
         
@@ -1204,7 +1210,7 @@ void CSOLNP::subnp(Eigen::MatrixBase<T2>& pars, Eigen::MatrixBase<T1>& yy_e, Eig
             Eigen::MatrixXd firstPart_e(1, 1 + neq + nineq);
             Eigen::RowVectorXd funv_e(1); funv_e[0] = funv;
             Eigen::RowVectorXd eqv_e(neq); eqv_e = fit.equality;
-            Eigen::RowVectorXd ineqv_e(nineq); ineqv_e= fit.inequality;
+            Eigen::RowVectorXd ineqv_e(nineq); ineqv_e= -fit.inequality;
 
             obj_constr_eval(funv_e, eqv_e, ineqv_e, firstPart_e, verbose);
             
@@ -1393,10 +1399,22 @@ void CSOLNP::obj_constr_eval(Eigen::MatrixBase<T2>& objVal, Eigen::MatrixBase<T2
 
     if (optimize_initial_inequality_constraints){
         double total = ineqval.array().min(0).sum();
-        fitVal << fabs(total) - 1e-4, eqval;
+        fitVal(0,0) = fabs(total) - 1e-4;
+	int dx=1;
+	for (int ix=0; ix < eqval.size(); ++ix,++dx) {
+		fitVal(0,dx) = eqval(0,ix);
+	}
     }
     else{
-	    fitVal << objVal, eqval, ineqval;
+	    fitVal.derived().resize(1,1+eqval.size()+ineqval.size());
+	    fitVal(0,0) = objVal(0);
+	    int dx=1;
+	    for (int ix=0; ix < eqval.size(); ++ix,++dx) {
+		    fitVal(0,dx) = eqval(0,ix);
+	    }
+	    for (int ix=0; ix < ineqval.size(); ++ix,++dx) {
+		    fitVal(0,dx) = ineqval(0,ix);
+	    }
     }
 
     if (!std::isfinite(fitVal.sum())) {
